@@ -8,18 +8,18 @@ import pytz
 import requests
 
 from .decorators import *
-from .exceptions import UnauthorizedError
+from .exceptions import UnauthorizedError, BadGatewayError
 from .mappings import Area, lookup_area, Indicator, lookup_balancing_zone, lookup_country, lookup_indicator, Country, \
     BalancingZone
 from .parsers import *
 
 __title__ = "entsog-py"
-__version__ = "0.8.1"
+__version__ = "0.9.1"
 __author__ = "nhcb"
 __license__ = "MIT"
 
 URL = 'https://transparency.entsog.eu/api/v1'
-
+OFFSET = 1000
 
 class EntsogRawClient:
     """
@@ -33,7 +33,7 @@ class EntsogRawClient:
 
     def __init__(
             self, session: Optional[requests.Session] = None,
-            retry_count: int = 1, retry_delay: int = 0,
+            retry_count: int = 3, retry_delay: int = 1.5,
             proxies: Optional[Dict] = None, timeout: Optional[int] = None):
         """
         Parameters
@@ -76,12 +76,11 @@ class EntsogRawClient:
             'limit': -1,
             'timeZone': 'UCT'
         }
-        params.update(base_params)
+        base_params.update(params)
 
         logging.debug(f'Performing request to {url} with params {params}')
 
         params = urllib.parse.urlencode(params, safe=',')  # ENTSOG uses comma-seperated values
-
         response = self.session.get(url=url, params=params,
                                     proxies=self.proxies, timeout=self.timeout)  # ,verify=False)
 
@@ -96,6 +95,8 @@ class EntsogRawClient:
                 elif response.status_code == 500:
                     # Gets a 500 error when the API is not available or no data is available
                     raise NoMatchingDataError
+                elif response.status_code == 502:
+                    raise BadGatewayError
 
             return response
 
@@ -177,12 +178,12 @@ class EntsogRawClient:
         -----------------
         """
 
-        response = self._base_request(endpoint='/connectionpoints')
+        response = self._base_request(endpoint='/connectionpoints', params = {})
 
         return response.text, response.url
 
     def query_operators(self,
-                        country_code: Union[Country, str],
+                        country_code: Union[Country, str] = None,
                         has_data: int = 1) -> str:
 
         """
@@ -946,8 +947,8 @@ class EntsogRawClient:
         response = self._base_request(endpoint='/aggregatedData', params=params)
 
         return response.text, response.url
-
-    def query_interruptions(self) -> str:
+    
+    def query_interruptions(self, start : pd.Timestamp, end : pd.Timestamp) -> str:
 
         """
         
@@ -1009,13 +1010,15 @@ class EntsogRawClient:
         "isArchived"
         -----------------
         """
-
-        response_text, response_url = self._base_offset_request(endpoint='/interruptions')
+        params = {
+            'from': self._datetime_to_str(start),
+            'to': self._datetime_to_str(end),
+        }
+        response_text, response_url = self._base_request(endpoint='/interruptions', params = params)
 
         return response_text, response_url
 
     def query_CMP_auction_premiums(self, start: pd.Timestamp, end: pd.Timestamp,
-                                   # country_code: Union[Country, str],
                                    period_type: str = 'day') -> str:
 
         """
@@ -1245,10 +1248,10 @@ class EntsogRawClient:
     def query_operational_data(self,
                                start: pd.Timestamp,
                                end: pd.Timestamp,
-                               operator: Optional[str] = None,
                                period_type: str = 'day',
                                indicators: Union[List[Indicator], List[str]] = None,
                                point_directions : Optional[List[str]] = None,
+                               offset : int = None,
                                ) -> str:
 
         """
@@ -1316,10 +1319,11 @@ class EntsogRawClient:
             'to': self._datetime_to_str(end),
             'periodType': period_type
         }
-
-        if operator is not None:
-            params['operatorKey'] = operator
-
+    
+        if offset is not None:
+            params['offset'] = offset
+            params['limit'] = OFFSET
+            
         if indicators is not None:
             decoded_indicators = []
             for indicator in indicators:
@@ -1368,7 +1372,7 @@ class EntsogPandasClient(EntsogRawClient):
         return data
 
     def query_operators(self,
-                        country_code: Union[Country, str],
+                        country_code: Union[Country, str] = None,
                         has_data: int = 1) -> pd.DataFrame:
 
         """
@@ -1384,7 +1388,8 @@ class EntsogPandasClient(EntsogRawClient):
         -------
         str
         """
-        country_code = lookup_country(country_code)
+        if country_code:
+            country_code = lookup_country(country_code)
         json, url = super(EntsogPandasClient, self).query_operators(
             country_code=country_code, has_data=has_data
         )
@@ -1648,8 +1653,9 @@ class EntsogPandasClient(EntsogRawClient):
         data['url'] = url
 
         return data
-
-    def query_interruptions(self, verbose: bool = True) -> pd.DataFrame:
+    
+    @day_limited
+    def query_interruptions(self, start : pd.Timestamp, end : pd.Timestamp, verbose : bool = False) -> pd.DataFrame:
 
         """
         Interruptions
@@ -1666,20 +1672,13 @@ class EntsogPandasClient(EntsogRawClient):
         pd.DataFrame
         """
 
-        json_list, url_list = super(EntsogPandasClient, self).query_interruptions()
-        frames = []
+        json, url = super(EntsogPandasClient, self).query_interruptions(start = start, end = end)
+        data = parse_interruptions(json, verbose)
+        data['url'] = url
 
-        for n_offset in range(1, len(json_list)):
-            json = json_list[n_offset]
-            url = url_list[n_offset]
-            frame = parse_interruptions(json, verbose)
-            frame['url'] = url
-            frames.append(frame)
-
-        return pd.DataFrame(frames)
+        return data
 
     def query_CMP_auction_premiums(self, start: pd.Timestamp, end: pd.Timestamp,
-                                   country_code: Union[Country, str],
                                    verbose: bool = True) -> pd.DataFrame:
 
         """
@@ -1696,9 +1695,8 @@ class EntsogPandasClient(EntsogRawClient):
         -------
         pd.DataFrame
         """
-        area = lookup_country(country_code)
         json, url = super(EntsogPandasClient, self).query_CMP_auction_premiums(
-            start=start, end=end, country_code=area
+            start=start, end=end
         )
         data = parse_CMP_auction_premiums(json, verbose)
         data['url'] = url
@@ -1706,7 +1704,6 @@ class EntsogPandasClient(EntsogRawClient):
         return data
 
     def query_CMP_unavailable_firm_capacity(self, start: pd.Timestamp, end: pd.Timestamp,
-                                            country_code: Union[Country, str],
                                             verbose: bool = True) -> pd.DataFrame:
 
         """
@@ -1723,9 +1720,8 @@ class EntsogPandasClient(EntsogRawClient):
         -------
         pd.DataFrame
         """
-        area = lookup_country(country_code)
         json, url = super(EntsogPandasClient, self).query_CMP_unavailable_firm_capacity(
-            start=start, end=end, country_code=area
+            start=start, end=end
         )
         data = parse_CMP_unavailable_firm_capacity(json, verbose)
         data['url'] = url
@@ -1734,7 +1730,6 @@ class EntsogPandasClient(EntsogRawClient):
 
     @week_limited
     def query_CMP_unsuccesful_requests(self, start: pd.Timestamp, end: pd.Timestamp,
-                                       country_code: Union[Country, str],
                                        verbose: bool = True) -> pd.DataFrame:
 
         """
@@ -1751,9 +1746,8 @@ class EntsogPandasClient(EntsogRawClient):
         -------
         pd.DataFrame
         """
-        area = lookup_country(country_code)
         json, url = super(EntsogPandasClient, self).query_CMP_unsuccesful_requests(
-            start=start, end=end, country_code=area
+            start=start, end=end
         )
         data = parse_CMP_unsuccesful_requests(json, verbose)
         data['url'] = url
@@ -1761,12 +1755,15 @@ class EntsogPandasClient(EntsogRawClient):
         return data
 
     @day_limited
+    @paginated
+    @documents_limited(OFFSET)
     def query_operational_data_all(self,
                                    start: pd.Timestamp,
                                    end: pd.Timestamp,
                                    period_type: str = 'day',
                                    indicators: Union[List[Indicator], List[str]] = ['physical_flow'],
-                                   verbose: bool = True) -> pd.DataFrame:
+                                   verbose: bool = True,
+                                   offset: int = 0) -> pd.DataFrame:
 
         """
         Operational data for all countries
@@ -1785,72 +1782,18 @@ class EntsogPandasClient(EntsogRawClient):
 
         """
 
-        if len(indicators) > 2:
-            raise NotImplementedError("Specify less than two indicators")
-        # Dangerous yet faster function, I noticed it can at least get two indicators based on both daily and hourly period type...
-        # For daily, it will take about 1.5m to obtain a month of data.
-
         json, url = super(EntsogPandasClient, self).query_operational_data(
             start=start,
             end=end,
             period_type=period_type,
-            indicators=indicators
+            indicators=indicators, 
+            offset = offset
         )
-
         data = parse_operational_data(json, verbose)
-
         data['url'] = url
 
         return data
 
-    def query_operational_data(self,
-                               start: pd.Timestamp,
-                               end: pd.Timestamp,
-                               country_code: Union[Area, str],
-                               period_type: str = 'day',
-                               indicators: Union[List[Indicator], List[str]] = ['physical_flow'],
-                               verbose: bool = True) -> pd.DataFrame:
-
-        """
-        
-        Nomination, Renominations, Allocations, Physical Flows, GCV,
-        Wobbe Index, Capacities, Interruptions, and CMP CMA
-
-        Parameters
-        ----------
-        start: pd.Timestamp
-        end: pd.Timestamp
-        country_code: Union[Area, str]
-        period_type: str
-        indicators: Union[List[Indicator],List[str]]
-        verbose: bool
-
-        Returns
-        -------
-        pd.DataFrame
-        """
-
-        area = lookup_area(country_code)
-        operators = list(area.value)
-
-        frames = []
-        for operator in operators:
-            try:
-                frame = self._query_operational_data(
-                    start=start,
-                    end=end,
-                    operator=operator,
-                    period_type=period_type,
-                    indicators=indicators,
-                    verbose=verbose)
-                frames.append(frame)
-            except Exception as e:
-                 print(f"Failure on operator {operator}: {e}")
-
-        result = pd.concat(frames)
-
-        return result
-
     @year_limited
     def query_operational_point_data(
         self,
@@ -1868,7 +1811,7 @@ class EntsogPandasClient(EntsogRawClient):
             period_type=period_type,
             indicators=indicators
         )
-
+        
         data = parse_operational_data(json_data, verbose)
         data['url'] = url
         return data
@@ -1894,112 +1837,4 @@ class EntsogPandasClient(EntsogRawClient):
 
         data = parse_operational_data(json_data, verbose)
         data['url'] = url
-        return data
-
-        # except Exception as e:
-        #     print(
-        #         f"Wrong or no data available for OPERATOR: {operator} and INDICATORS:{indicators}. " +
-        #         f"Due to the poor structure and documentation of the data, this occurs frequently: {e}")
-        #     return None
-
-    def get_operational_aggregates(
-            self,
-            start: pd.Timestamp,
-            end: pd.Timestamp,
-            country_code: Union[Area, str],
-            period_type: str = 'day',
-            indicators: Union[List[Indicator], List[str]] = None) -> pd.DataFrame:
-
-        raise NotImplementedError("Function not implemented anymore")
-
-        if self._operator_point_directions is None:
-            self._operator_point_directions = self.query_operator_point_directions()
-
-        # Get the data
-        data = self.query_operational_data(
-            start=start,
-            end=end,
-            country_code=country_code,
-            period_type=period_type,
-            indicators=indicators
-        )
-        # Merge the data
-        merged = pd.merge(
-            data,
-            self._operator_point_directions,
-            left_on=['operator_key', 'point_key'],
-            right_on=['operator_key', 'point_key'],
-            suffixes=['', '_static'],
-            how='left'
-        )
-
-        columns = [
-            "indicator",
-            "period_type",
-            "period_from",
-            "period_to",
-            "operator_key",
-            "tso_eic_code",
-            "operator_label",
-            "point_key",
-            "point_label",
-            "tso_item_identifier",
-            "last_update_date_time",
-            "direction_key",
-            "value",
-            "flow_status",
-            "is_cmp_relevant",
-            "booking_platform_key",
-            "point_type",
-            "is_pipe_in_pipe",
-            "tp_tso_item_label",
-            "last_update_date_time_static",
-            "virtual_reverse_flow",
-            "tso_country",
-            "tso_balancing_zone",
-            "cross_border_point_type",
-            "eu_relationship",
-            "connected_operators",
-            "adjacent_operator_key",
-            "adjacent_country",
-            "adjacent_zones"  # e.g. Switzerland
-        ]
-
-        subset = merged[columns]
-
-        return subset
-
-    def get_grouped_operational_aggregates(
-            self,
-            start: pd.Timestamp,
-            end: pd.Timestamp,
-            country_code: Union[Area, str],
-            period_type: str = 'day',
-            indicators: Union[List[Indicator], List[str]] = None,
-            groups: List[str] = ['point', 'operator', 'country', 'balancing_zone', 'region'],
-            entry_exit: bool = False) -> dict:
-
-        raise NotImplementedError("Function not implemented anymore")
-
-        if list(set(groups).difference(['point', 'operator', 'country', 'balancing_zone', 'region'])):
-            raise ValueError(f'{groups} contain not a valid group type, please specify any of the following: {groups}.')
-
-        data = self.get_operational_aggregates(
-            start=start,
-            end=end,
-            country_code=country_code,
-            period_type=period_type,
-            indicators=indicators
-        )
-        data.to_csv(f"data/get_operational_aggregates_{country_code}.csv", sep=';')
-
-        result = {}
-        for group in groups:
-            small = parse_grouped_operational_aggregates(
-                data=data,
-                group_type=group,
-                entry_exit=entry_exit
-            )
-            result[group] = small
-
-        return result
+        return data
\ No newline at end of file
