@@ -1,6 +1,8 @@
 import bs4
 import pandas as pd
 import json
+
+from entsog.exceptions import NoMatchingDataError
 from .mappings import REGIONS
 from .misc import to_snake_case
 
@@ -30,10 +32,14 @@ def parse_operational_data(json_text: str, verbose: bool):
                'capacity_type',
                'last_update_date_time',
                'item_remarks', 'general_remarks']
-    if verbose:
-        return data
+    
+    if not data.empty:
+        if verbose:
+            return data
+        else:
+            return data[columns]
     else:
-        return data[columns]
+        raise NoMatchingDataError('No matching data found')
 
 
 def parse_CMP_unsuccesful_requests(json_text: str, verbose: bool):
@@ -46,10 +52,13 @@ def parse_CMP_unsuccesful_requests(json_text: str, verbose: bool):
                'occurence_count',
                'item_remarks', 'general_remarks']
 
-    if verbose:
-        return data
+    if not data.empty:
+        if verbose:
+            return data
+        else:
+            return data[columns]
     else:
-        return data[columns]
+        raise NoMatchingDataError('No matching data found')
 
 
 def parse_CMP_unavailable_firm_capacity(json_text: str, verbose: bool):
@@ -61,10 +70,13 @@ def parse_CMP_unavailable_firm_capacity(json_text: str, verbose: bool):
                'last_update_date_time',
                'item_remarks', 'general_remarks']
 
-    if verbose:
-        return data
+    if not data.empty:
+        if verbose:
+            return data
+        else:
+            return data[columns]
     else:
-        return data[columns]
+        raise NoMatchingDataError('No matching data found')
 
 
 def parse_CMP_auction_premiums(json_text: str, verbose: bool):
@@ -77,10 +89,13 @@ def parse_CMP_auction_premiums(json_text: str, verbose: bool):
                'last_update_date_time',
                'item_remarks', 'general_remarks']
 
-    if verbose:
-        return data
+    if not data.empty:
+        if verbose:
+            return data
+        else:
+            return data[columns]
     else:
-        return data[columns]
+        raise NoMatchingDataError('No matching data found')
 
 
 def parse_interruptions(json_text: str, verbose: bool):
@@ -92,10 +107,13 @@ def parse_interruptions(json_text: str, verbose: bool):
                'last_update_date_time',
                'item_remarks', 'general_remarks']
 
-    if verbose:
-        return data
+    if not data.empty:
+        if verbose:
+            return data
+        else:
+            return data[columns]
     else:
-        return data[columns]
+        raise NoMatchingDataError('No matching data found')
 
 
 # TODO: implement melt...
@@ -123,26 +141,28 @@ def parse_tariffs_sim(json_text: str, verbose: bool, melt: bool):
                'item_remarks',
                'general_remarks']
 
-    if verbose:
-        return data
+    if not data.empty:
+        if verbose:
+            return data
+        else:
+            return data[columns]
     else:
-        return data[columns]
+        raise NoMatchingDataError('No matching data found')
 
 
 def parse_tariffs(json_text: str, verbose: bool, melt: bool):
     # https://transparency.entsog.eu/api/v1/tariffsfulls
 
     data = parse_general(json_text)
-
     renamed_columns = {
-        'applicable_tariff_per_local_currency_k_wh_d_value': 'applicable_tariff_per_local_currency_kwh_d_value',
-        'applicable_tariff_per_local_currency_k_wh_d_unit': 'applicable_tariff_per_local_currency_kwh_d_unit',
-        'applicable_tariff_per_local_currency_k_wh_h_value': 'applicable_tariff_per_local_currency_kwh_h_value',
-        'applicable_tariff_per_local_currency_k_wh_h_unit': 'applicable_tariff_per_local_currency_kwh_h_unit',
-        'applicable_tariff_per_eurk_wh_d_unit': 'applicable_tariff_per_eur_kwh_d_unit',
-        'applicable_tariff_per_eurk_wh_d_value': 'applicable_tariff_per_eur_kwh_d_value',
-        'applicable_tariff_per_eurk_wh_h_unit': 'applicable_tariff_per_eur_kwh_h_unit',
-        'applicable_tariff_per_eurk_wh_h_value': 'applicable_tariff_per_eur_kwh_h_value',
+        'applicable_tariff_per_local_currency_kwh_d_value': 'applicable_tariff_per_local_currency_kwh_d_value',
+        'applicable_tariff_per_local_currency_kwh_d_unit': 'applicable_tariff_per_local_currency_kwh_d_unit',
+        'applicable_tariff_per_local_currency_kwh_h_value': 'applicable_tariff_per_local_currency_kwh_h_value',
+        'applicable_tariff_per_local_currency_kwh_h_unit': 'applicable_tariff_per_local_currency_kwh_h_unit',
+        'applicable_tariff_per_eurkwh_d_unit': 'applicable_tariff_per_eur_kwh_d_unit',
+        'applicable_tariff_per_eurkwh_d_value': 'applicable_tariff_per_eur_kwh_d_value',
+        'applicable_tariff_per_eurkwh_h_unit': 'applicable_tariff_per_eur_kwh_h_unit',
+        'applicable_tariff_per_eurkwh_h_value': 'applicable_tariff_per_eur_kwh_h_value',
         'applicable_commodity_tariff_local_currency': 'applicable_commodity_tariff_local_currency'
     }
 
@@ -183,10 +203,9 @@ def parse_tariffs(json_text: str, verbose: bool, melt: bool):
         'item_remarks',
         'general_remarks']
 
-    if verbose:
-        columns = data.columns  # Pick all
-        data = data[columns]
-    else:
+    if verbose and not melt:
+        return data
+    elif not verbose or melt:
         data = data[columns]
 
     if melt:
@@ -206,7 +225,7 @@ def parse_tariffs(json_text: str, verbose: bool, melt: bool):
             "applicable_tariff_in_common_unit_unit",
         ]
 
-        id_columns = list(set(columns) - set(melt_columns_unit) - set(melt_columns_value))
+        id_columns = list(set(columns) - set(melt_columns_unit + melt_columns_value))
 
         data_value = pd.melt(
             data,
@@ -215,8 +234,9 @@ def parse_tariffs(json_text: str, verbose: bool, melt: bool):
             var_name='variable',
             value_name='value'
         )
-
+        
         data_value['variable'] = data_value["variable"].str.replace("_value$", "")
+        
         data_unit = pd.melt(
             data,
             id_vars=id_columns,
@@ -226,10 +246,11 @@ def parse_tariffs(json_text: str, verbose: bool, melt: bool):
         )
         data_unit['variable'] = data_unit["variable"].str.replace("_unit$", "")
 
-        merge_columns = id_columns.append('variable')
-
-        data_pivot = data_value.merge(data_unit, on=merge_columns)
+        # Append with variable
+        id_columns.append('variable')
 
+        data_pivot = pd.merge(data_value, data_unit, on=id_columns)
+        
         data_pivot['variable'] = data_pivot['variable'].str.extract(r'(local_currency|eur|common_unit)')
         data_pivot['currency'] = data_pivot['code'].str.extract(r'^(.*?)\/')  # ^(.*?)\/ LINE START
         data_pivot['unit'] = data_pivot['code'].str.extract(
@@ -282,10 +303,13 @@ def parse_aggregate_data(
         'period_from', 'period_to', 'period_type', 'direction_key', 'indicator',
         'unit', 'value']
 
-    if verbose:
-        return data
+    if not data.empty:
+        if verbose:
+            return data
+        else:
+            return data[columns]
     else:
-        return data[columns]
+        raise NoMatchingDataError('No matching data found')
 
 
 # Legacy stuff
@@ -568,20 +592,3 @@ def parse_grouped_operational_aggregates(
         )
 
     return df.reset_index()
-
-
-def _extract_timeseries(xml_text):
-    """
-    Parameters
-    ----------
-    xml_text : str
-
-    Yields
-    -------
-    bs4.element.tag
-    """
-    if not xml_text:
-        return
-    soup = bs4.BeautifulSoup(xml_text, 'html.parser')
-    for timeseries in soup.find_all('timeseries'):
-        yield timeseries
